@@ -1,4 +1,4 @@
-/* $Id: ipfrdr.c,v 1.19 2016/12/16 09:11:19 nanard Exp $ */
+/* $Id: ipfrdr.c,v 1.21 2020/12/20 17:43:40 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
  * (c) 2007 Darren Reed
@@ -59,7 +59,7 @@ struct file;
 #include <string.h>
 #include <unistd.h>
 
-#include "../config.h"
+#include "config.h"
 #include "netinet/ipl.h"
 #include "netinet/ip_compat.h"
 #include "netinet/ip_fil.h"
@@ -804,7 +804,7 @@ get_portmappings_in_range(unsigned short startport, unsigned short endport,
 	return array;
 }
 
-/* update the port mapping internal port, decription and timestamp */
+/* update the port mapping internal port, description and timestamp */
 int
 update_portmapping(const char * ifname, unsigned short eport, int proto,
                    unsigned short iport, const char * desc,
@@ -817,7 +817,7 @@ update_portmapping(const char * ifname, unsigned short eport, int proto,
 	return -1;
 }
 
-/* update the port mapping decription and timestamp */
+/* update the port mapping description and timestamp */
 int
 update_portmapping_desc_timestamp(const char * ifname,
                    unsigned short eport, int proto,
