@@ -1,8 +1,8 @@
-/* $Id: minissdp.c,v 1.108 2024/05/16 00:15:56 nanard Exp $ */
+/* $Id: minissdp.c,v 1.110 2025/01/25 21:16:20 nanard Exp $ */
 /* vim: tabstop=4 shiftwidth=4 noexpandtab
  * MiniUPnP project
  * http://miniupnp.free.fr/ or https://miniupnp.tuxfamily.org/
- * (c) 2006-2024 Thomas Bernard
+ * (c) 2006-2025 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -47,10 +47,24 @@
 /* SSDP ip/port */
 #define SSDP_PORT (1900)
 #define SSDP_MCAST_ADDR ("239.255.255.250")
+
+#ifdef ENABLE_IPV6
 #define LL_SSDP_MCAST_ADDR "FF02::C"
 #define SL_SSDP_MCAST_ADDR "FF05::C"
 #define GL_SSDP_MCAST_ADDR "FF0E::C"
 
+/* UDA 1.1 AnnexA and UDA 2.0 only allow/define the use of
+ * Link-Local and Site-Local multicast scopes */
+static struct { const char * p1, * p2; } const mcast_addrs[] = {
+	{ LL_SSDP_MCAST_ADDR, "[" LL_SSDP_MCAST_ADDR "]" },	/* Link Local */
+	{ SL_SSDP_MCAST_ADDR, "[" SL_SSDP_MCAST_ADDR "]" },	/* Site Local */
+#ifndef UPNP_STRICT
+	{ GL_SSDP_MCAST_ADDR, "[" GL_SSDP_MCAST_ADDR "]" },	/* Global */
+#endif /* ! UPNP_STRICT */
+	{ NULL, NULL }
+};
+#endif /* ENABLE_IPV6 */
+
 /* AddMulticastMembership()
  * param s			socket
  * param lan_addr	lan address
@@ -220,7 +234,11 @@ OpenAndConfSSDPReceiveSocket(int ipv6)
 #endif /* IP_PKTINFO */
 #if defined(ENABLE_IPV6) && defined(IPV6_RECVPKTINFO)
 	if(ipv6) {
+#ifdef IPPROTO_IPV6
+		if(setsockopt(s, IPPROTO_IPV6, IPV6_RECVPKTINFO, &on, sizeof(on)) < 0)
+#else
 		if(setsockopt(s, IPPROTO_IP, IPV6_RECVPKTINFO, &on, sizeof(on)) < 0)
+#endif
 		{
 			syslog(LOG_WARNING, "setsockopt(udp, IPV6_RECVPKTINFO): %m");
 		}
@@ -402,7 +420,19 @@ OpenAndConfSSDPNotifySocketIPv6(struct lan_addr_s * lan_addr)
 		close(s);
 		return -1;
 	}
-
+#if defined(SO_BINDTODEVICE) && !defined(MULTIPLE_EXTERNAL_IP)
+	/* One and only one LAN interface */
+	if(lan_addrs.lh_first != NULL && lan_addrs.lh_first->list.le_next == NULL
+	   && lan_addrs.lh_first->ifname[0] != '\0')
+	{
+		if(setsockopt(s, SOL_SOCKET, SO_BINDTODEVICE,
+		              lan_addrs.lh_first->ifname,
+		              strlen(lan_addrs.lh_first->ifname) + 1) < 0)
+			syslog(LOG_WARNING, "%s: setsockopt(udp6, SO_BINDTODEVICE, %s): %m",
+			       "OpenAndConfSSDPNotifySocketIPv6",
+			       lan_addrs.lh_first->ifname);
+	}
+#endif /* defined(SO_BINDTODEVICE) && !defined(MULTIPLE_EXTERNAL_IP) */
 	/* bind() socket before using sendto() is not mandatory
 	 * (sendto() will implicitly bind the socket when called on
 	 * an unbound socket)
@@ -742,15 +772,6 @@ SendSSDPNotifies(int s, const char * host, unsigned short http_port,
 {
 #ifdef ENABLE_IPV6
 	struct sockaddr_storage sockname;
-	/* UDA 1.1 AnnexA and UDA 2.0 only allow/define the use of
-	 * Link-Local and Site-Local multicast scopes */
-	static struct { const char * p1, * p2; } const mcast_addrs[] =
-		{ { LL_SSDP_MCAST_ADDR, "[" LL_SSDP_MCAST_ADDR "]" },	/* Link Local */
-		  { SL_SSDP_MCAST_ADDR, "[" SL_SSDP_MCAST_ADDR "]" },	/* Site Local */
-#ifndef UPNP_STRICT
-		  { GL_SSDP_MCAST_ADDR, "[" GL_SSDP_MCAST_ADDR "]" },	/* Global */
-#endif /* ! UPNP_STRICT */
-		  { NULL, NULL } };
 	int j;
 #else /* ENABLE_IPV6 */
 	struct sockaddr_in sockname;
@@ -1421,7 +1442,7 @@ SendSSDPbyebye(int s, const struct sockaddr * dest, socklen_t destlen,
 	n = sendto_or_schedule(s, bufr, l, 0, dest, destlen);
 	if(n < 0)
 	{
-		syslog(LOG_ERR, "sendto(udp_shutdown=%d): %m", s);
+		syslog(LOG_ERR, "sendto(udp_shutdown=%d) to %s: %m", s, dest_str);
 		return -1;
 	}
 	else if(n != l)
@@ -1443,6 +1464,7 @@ SendSSDPGoodbye(int * sockets, int n_sockets)
 	struct sockaddr * sockname;
 	socklen_t socknamelen;
 	int ipv6 = 0;
+	int k;
 #endif
 	int i, j;
 	char ver_str[4];
@@ -1457,7 +1479,6 @@ SendSSDPGoodbye(int * sockets, int n_sockets)
 	memset(&sockname6, 0, sizeof(struct sockaddr_in6));
 	sockname6.sin6_family = AF_INET6;
 	sockname6.sin6_port = htons(SSDP_PORT);
-	inet_pton(AF_INET6, LL_SSDP_MCAST_ADDR, &(sockname6.sin6_addr));
 #else
 	dest_str = SSDP_MCAST_ADDR;
 #endif
@@ -1468,36 +1489,25 @@ SendSSDPGoodbye(int * sockets, int n_sockets)
 			continue;
 #ifdef ENABLE_IPV6
 		ipv6 = j & 1;
-		if(ipv6) {
-			dest_str = "[" LL_SSDP_MCAST_ADDR "]";
-			sockname = (struct sockaddr *)&sockname6;
-			socknamelen = sizeof(struct sockaddr_in6);
-		} else {
-			dest_str = SSDP_MCAST_ADDR;
-			sockname = (struct sockaddr *)&sockname4;
-			socknamelen = sizeof(struct sockaddr_in);
-		}
-#endif
-	    for(i=0; known_service_types[i].s; i++)
-	    {
-			if(i==0)
-				ver_str[0] = '\0';
-			else
-				snprintf(ver_str, sizeof(ver_str), "%d", known_service_types[i].version);
-			ret += SendSSDPbyebye(sockets[j],
-#ifdef ENABLE_IPV6
-			                      sockname, socknamelen,
+		for(k = 0; (mcast_addrs[k].p1 != 0 && ipv6) || k < 1; k++) {
+			if(ipv6) {
+				dest_str = mcast_addrs[k].p2;
+				inet_pton(AF_INET6, mcast_addrs[k].p1, &(sockname6.sin6_addr));
+				sockname = (struct sockaddr *)&sockname6;
+				socknamelen = sizeof(struct sockaddr_in6);
+			} else {
+				dest_str = SSDP_MCAST_ADDR;
+				sockname = (struct sockaddr *)&sockname4;
+				socknamelen = sizeof(struct sockaddr_in);
+			}
 #else
-			                      (struct sockaddr *)&sockname4, sizeof(struct sockaddr_in),
+		{
 #endif
-			                      dest_str,
-			                      known_service_types[i].s, ver_str,	/* NT: */
-			                      known_service_types[i].uuid, "::",
-			                      known_service_types[i].s); /* ver_str, USN: */
-			if(i > 0 &&	/* only known_service_types[0].s is shorter than "urn:schemas-upnp-org:device" */
-			   0==memcmp(known_service_types[i].s,
-			             "urn:schemas-upnp-org:device", sizeof("urn:schemas-upnp-org:device")-1))
-			{
+		    for(i=0; known_service_types[i].s; i++) {
+				if(i==0)
+					ver_str[0] = '\0';
+				else
+					snprintf(ver_str, sizeof(ver_str), "%d", known_service_types[i].version);
 				ret += SendSSDPbyebye(sockets[j],
 #ifdef ENABLE_IPV6
 				                      sockname, socknamelen,
@@ -1505,8 +1515,23 @@ SendSSDPGoodbye(int * sockets, int n_sockets)
 				                      (struct sockaddr *)&sockname4, sizeof(struct sockaddr_in),
 #endif
 				                      dest_str,
-				                      known_service_types[i].uuid, "",	/* NT: */
-				                      known_service_types[i].uuid, "", ""); /* ver_str, USN: */
+				                      known_service_types[i].s, ver_str,	/* NT: */
+				                      known_service_types[i].uuid, "::",
+				                      known_service_types[i].s); /* ver_str, USN: */
+				if(i > 0 &&	/* only known_service_types[0].s is shorter than "urn:schemas-upnp-org:device" */
+				   0==memcmp(known_service_types[i].s,
+				             "urn:schemas-upnp-org:device", sizeof("urn:schemas-upnp-org:device")-1))
+				{
+					ret += SendSSDPbyebye(sockets[j],
+#ifdef ENABLE_IPV6
+					                      sockname, socknamelen,
+#else
+					                      (struct sockaddr *)&sockname4, sizeof(struct sockaddr_in),
+#endif
+					                      dest_str,
+					                      known_service_types[i].uuid, "",	/* NT: */
+					                      known_service_types[i].uuid, "", ""); /* ver_str, USN: */
+				}
 			}
 		}
 	}
