@@ -1,11 +1,12 @@
-/* $Id: pfpinhole.c,v 1.29 2020/05/10 22:22:50 nanard Exp $ */
+/* $Id: pfpinhole.c,v 1.31 2024/06/22 16:48:54 nanard Exp $ */
 /* vim: tabstop=4 shiftwidth=4 noexpandtab
  * MiniUPnP project
  * http://miniupnp.free.fr/ or https://miniupnp.tuxfamily.org/
- * (c) 2012-2020 Thomas Bernard
+ * (c) 2012-2024 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
+#include "config.h"
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/param.h>
@@ -28,12 +29,15 @@
 #include <syslog.h>
 #include <stdio.h>
 #include <stdlib.h>
+#ifdef USE_LIBPFCTL
+#include <libpfctl.h>
+#endif
 
-#include "config.h"
 #include "pfpinhole.h"
 #include "../upnpglobalvars.h"
 #include "../macros.h"
 #include "../upnputils.h"
+#include "rtickets.h"
 
 /* the pass rules created by add_pinhole() are as follow :
  *
@@ -55,13 +59,15 @@ static int next_uid = 1;
 #define PINEHOLE_LABEL_FORMAT "pinhole-%d ts-%u: %s"
 #define PINEHOLE_LABEL_FORMAT_SKIPDESC "pinhole-%d ts-%u: %*s"
 
+#define RULE (pr.rule)
+
 int add_pinhole(const char * ifname,
                 const char * rem_host, unsigned short rem_port,
                 const char * int_client, unsigned short int_port,
                 int proto, const char * desc, unsigned int timestamp)
 {
 	int uid;
-	struct pfioc_rule pcr;
+	struct pfioc_rule pr;
 #ifndef PF_NEWSTYLE
 	struct pfioc_pooladdr pp;
 #endif
@@ -70,8 +76,8 @@ int add_pinhole(const char * ifname,
 		syslog(LOG_ERR, "pf device is not open");
 		return -1;
 	}
-	memset(&pcr, 0, sizeof(pcr));
-	strlcpy(pcr.anchor, anchor_name, MAXPATHLEN);
+	memset(&pr, 0, sizeof(pr));
+	strlcpy(pr.anchor, anchor_name, MAXPATHLEN);
 
 #ifndef PF_NEWSTYLE
 	memset(&pp, 0, sizeof(pp));
@@ -80,74 +86,74 @@ int add_pinhole(const char * ifname,
 		syslog(LOG_ERR, "ioctl(dev, DIOCBEGINADDRS, ...): %m");
 		return -1;
 	} else {
-		pcr.pool_ticket = pp.ticket;
+		pr.pool_ticket = pp.ticket;
 #else
 	{
 #endif
-		pcr.rule.direction = PF_IN;
-		pcr.rule.action = PF_PASS;
-		pcr.rule.af = AF_INET6;
+		RULE.direction = PF_IN;
+		RULE.action = PF_PASS;
+		RULE.af = AF_INET6;
 #ifdef PF_NEWSTYLE
-		pcr.rule.nat.addr.type = PF_ADDR_NONE;
-		pcr.rule.rdr.addr.type = PF_ADDR_NONE;
+		RULE.nat.addr.type = PF_ADDR_NONE;
+		RULE.rdr.addr.type = PF_ADDR_NONE;
 #endif
 #ifdef USE_IFNAME_IN_RULES
 		if(ifname)
-			strlcpy(pcr.rule.ifname, ifname, IFNAMSIZ);
+			strlcpy(RULE.ifname, ifname, IFNAMSIZ);
 #endif
-		pcr.rule.proto = proto;
+		RULE.proto = proto;
 
-		pcr.rule.quick = 1;/*(GETFLAG(PFNOQUICKRULESMASK))?0:1;*/
-		pcr.rule.log = (GETFLAG(LOGPACKETSMASK))?1:0;	/*logpackets;*/
+		RULE.quick = 1;/*(GETFLAG(PFNOQUICKRULESMASK))?0:1;*/
+		RULE.log = (GETFLAG(LOGPACKETSMASK))?1:0;	/*logpackets;*/
 /* see the discussion on the forum :
  * http://miniupnp.tuxfamily.org/forum/viewtopic.php?p=638 */
-		pcr.rule.flags = TH_SYN;
-		pcr.rule.flagset = (TH_SYN|TH_ACK);
+		RULE.flags = TH_SYN;
+		RULE.flagset = (TH_SYN|TH_ACK);
 #ifdef PFRULE_HAS_RTABLEID
-		pcr.rule.rtableid = -1;	/* first appeared in OpenBSD 4.0 */
+		RULE.rtableid = -1;	/* first appeared in OpenBSD 4.0 */
 #endif
 #ifdef PFRULE_HAS_ONRDOMAIN
-		pcr.rule.onrdomain = -1;	/* first appeared in OpenBSD 5.0 */
+		RULE.onrdomain = -1;	/* first appeared in OpenBSD 5.0 */
 #endif
-		pcr.rule.keep_state = 1;
+		RULE.keep_state = 1;
 		uid = next_uid;
-		snprintf(pcr.rule.label, PF_RULE_LABEL_SIZE,
+		snprintf(RULE.label, PF_RULE_LABEL_SIZE,
 		         PINEHOLE_LABEL_FORMAT, uid, timestamp, desc);
 		if(queue)
-			strlcpy(pcr.rule.qname, queue, PF_QNAME_SIZE);
+			strlcpy(RULE.qname, queue, PF_QNAME_SIZE);
 		if(tag)
-			strlcpy(pcr.rule.tagname, tag, PF_TAG_NAME_SIZE);
+			strlcpy(RULE.tagname, tag, PF_TAG_NAME_SIZE);
 
 		if(rem_port) {
-			pcr.rule.src.port_op = PF_OP_EQ;
-			pcr.rule.src.port[0] = htons(rem_port);
+			RULE.src.port_op = PF_OP_EQ;
+			RULE.src.port[0] = htons(rem_port);
 		}
 		if(rem_host && rem_host[0] != '\0' && rem_host[0] != '*') {
-			pcr.rule.src.addr.type = PF_ADDR_ADDRMASK;
-			if(inet_pton(AF_INET6, rem_host, &pcr.rule.src.addr.v.a.addr.v6) != 1) {
+			RULE.src.addr.type = PF_ADDR_ADDRMASK;
+			if(inet_pton(AF_INET6, rem_host, &RULE.src.addr.v.a.addr.v6) != 1) {
 				syslog(LOG_ERR, "inet_pton(%s) failed", rem_host);
 			}
-			memset(&pcr.rule.src.addr.v.a.mask.addr8, 255, 16);
+			memset(&RULE.src.addr.v.a.mask.addr8, 255, 16);
 		}
 
-		pcr.rule.dst.port_op = PF_OP_EQ;
-		pcr.rule.dst.port[0] = htons(int_port);
-		pcr.rule.dst.addr.type = PF_ADDR_ADDRMASK;
-		if(inet_pton(AF_INET6, int_client, &pcr.rule.dst.addr.v.a.addr.v6) != 1) {
+		RULE.dst.port_op = PF_OP_EQ;
+		RULE.dst.port[0] = htons(int_port);
+		RULE.dst.addr.type = PF_ADDR_ADDRMASK;
+		if(inet_pton(AF_INET6, int_client, &RULE.dst.addr.v.a.addr.v6) != 1) {
 			syslog(LOG_ERR, "inet_pton(%s) failed", int_client);
 		}
-		memset(&pcr.rule.dst.addr.v.a.mask.addr8, 255, 16);
+		memset(&RULE.dst.addr.v.a.mask.addr8, 255, 16);
 
 		if(ifname)
-			strlcpy(pcr.rule.ifname, ifname, IFNAMSIZ);
+			strlcpy(RULE.ifname, ifname, IFNAMSIZ);
 
-		pcr.action = PF_CHANGE_GET_TICKET;
-		if(ioctl(dev, DIOCCHANGERULE, &pcr) < 0) {
+		pr.action = PF_CHANGE_GET_TICKET;
+		if(ioctl(dev, DIOCCHANGERULE, &pr) < 0) {
 			syslog(LOG_ERR, "ioctl(dev, DIOCCHANGERULE, ...) PF_CHANGE_GET_TICKET: %m");
 			return -1;
 		} else {
-			pcr.action = PF_CHANGE_ADD_TAIL;
-			if(ioctl(dev, DIOCCHANGERULE, &pcr) < 0) {
+			pr.action = PF_CHANGE_ADD_TAIL;
+			if(ioctl(dev, DIOCCHANGERULE, &pr) < 0) {
 				syslog(LOG_ERR, "ioctl(dev, DIOCCHANGERULE, ...) PF_CHANGE_ADD_TAIL: %m");
 				return -1;
 			}
@@ -167,9 +173,18 @@ int find_pinhole(const char * ifname,
                  char *desc, int desc_len, unsigned int * timestamp)
 {
 	int uid;
-	unsigned int ts;
+	unsigned int ts, tnum;
 	int i, n;
+#undef RULE
+#ifdef USE_LIBPFCTL
+	struct pfctl_rules_info ri;
+	struct pfctl_rule rule;
+#define RULE (rule)
+	char anchor_call[MAXPATHLEN] = "";
+#else /* USE_LIBPFCTL */
 	struct pfioc_rule pr;
+#define RULE (pr.rule)
+#endif /* USE_LIBPFCTL */
 	struct in6_addr saddr;
 	struct in6_addr daddr;
 	UNUSED(ifname);
@@ -184,48 +199,96 @@ int find_pinhole(const char * ifname,
 		memset(&saddr, 0, sizeof(struct in6_addr));
 	}
 	inet_pton(AF_INET6, int_client, &daddr);
+#ifdef USE_LIBPFCTL
+	if(pfctl_get_rules_info(dev, &ri, PF_PASS, anchor_name) < 0)
+	{
+		syslog(LOG_ERR, "pfctl_get_rules_info: %m");
+		return -1;
+	}
+	n = ri.nr;
+#ifdef PF_RELEASETICKETS
+	tnum = ri.ticket;
+#endif /* PF_RELEASETICKETS */
+#else /* USE_LIBPFCTL */
 	memset(&pr, 0, sizeof(pr));
 	strlcpy(pr.anchor, anchor_name, MAXPATHLEN);
 #ifndef PF_NEWSTYLE
-	pr.rule.action = PF_PASS;
+	RULE.action = PF_PASS;
 #endif
 	if(ioctl(dev, DIOCGETRULES, &pr) < 0) {
 		syslog(LOG_ERR, "ioctl(dev, DIOCGETRULES, ...): %m");
 		return -1;
 	}
 	n = pr.nr;
+#ifdef PF_RELEASETICKETS
+	tnum = pr.ticket;
+#endif /* PF_RELEASETICKETS */
+#endif /* USE_LIBPFCTL */
 	for(i=0; i<n; i++) {
+#ifdef USE_LIBPFCTL
+		if(pfctl_get_rule(dev, i, ri.ticket, anchor_name, PF_PASS, &rule, anchor_call) < 0)
+		{
+			syslog(LOG_ERR, "pfctl_get_rule: %m");
+			release_ticket(dev, tnum);
+			return -1;
+		}
+#else /* USE_LIBPFCTL */
 		pr.nr = i;
 		if(ioctl(dev, DIOCGETRULE, &pr) < 0) {
 			syslog(LOG_ERR, "ioctl(dev, DIOCGETRULE): %m");
+			release_ticket(dev, tnum);
 			return -1;
 		}
-		if((proto == pr.rule.proto) && (rem_port == ntohs(pr.rule.src.port[0]))
-		   && (0 == memcmp(&saddr, &pr.rule.src.addr.v.a.addr.v6, sizeof(struct in6_addr)))
-		   && (int_port == ntohs(pr.rule.dst.port[0])) &&
-		   (0 == memcmp(&daddr, &pr.rule.dst.addr.v.a.addr.v6, sizeof(struct in6_addr)))) {
-			if(sscanf(pr.rule.label, PINEHOLE_LABEL_FORMAT_SKIPDESC, &uid, &ts) != 2) {
-				syslog(LOG_DEBUG, "rule with label '%s' is not a IGD pinhole", pr.rule.label);
+#endif /* USE_LIBPFCTL */
+		if((proto == RULE.proto) && (rem_port == ntohs(RULE.src.port[0]))
+		   && (0 == memcmp(&saddr, &RULE.src.addr.v.a.addr.v6, sizeof(struct in6_addr)))
+		   && (int_port == ntohs(RULE.dst.port[0])) &&
+		   (0 == memcmp(&daddr, &RULE.dst.addr.v.a.addr.v6, sizeof(struct in6_addr)))) {
+#ifdef USE_LIBPFCTL
+			if(sscanf(RULE.label[0], PINEHOLE_LABEL_FORMAT_SKIPDESC, &uid, &ts) != 2) {
+				syslog(LOG_DEBUG, "rule with label '%s' is not a IGD pinhole", RULE.label[0]);
 				continue;
 			}
+#else /* USE_LIBPFCTL */
+			if(sscanf(RULE.label, PINEHOLE_LABEL_FORMAT_SKIPDESC, &uid, &ts) != 2) {
+				syslog(LOG_DEBUG, "rule with label '%s' is not a IGD pinhole", RULE.label);
+				continue;
+			}
+#endif /* USE_LIBPFCTL */
 			if(timestamp) *timestamp = ts;
 			if(desc) {
-				char * p = strchr(pr.rule.label, ':');
+#ifdef USE_LIBPFCTL
+				char * p = strchr(RULE.label[0], ':');
+#else /* USE_LIBPFCTL */
+				char * p = strchr(RULE.label, ':');
+#endif /* USE_LIBPFCTL */
 				if(p) {
 					p += 2;
 					strlcpy(desc, p, desc_len);
 				}
 			}
+			release_ticket(dev, tnum);
 			return uid;
 		}
 	}
+	release_ticket(dev, tnum);
 	return -2;
 }
 
 int delete_pinhole(unsigned short uid)
 {
 	int i, n;
+	unsigned int tnum;
+#undef RULE
+#ifdef USE_LIBPFCTL
+	struct pfctl_rules_info ri;
+	struct pfctl_rule rule;
+#define RULE (rule)
+	char anchor_call[MAXPATHLEN] = "";
+#else /* USE_LIBPFCTL */
 	struct pfioc_rule pr;
+#define RULE (pr.rule)
+#endif /* USE_LIBPFCTL */
 	char label_start[PF_RULE_LABEL_SIZE];
 	char tmp_label[PF_RULE_LABEL_SIZE];
 
@@ -235,39 +298,75 @@ int delete_pinhole(unsigned short uid)
 	}
 	snprintf(label_start, sizeof(label_start),
 	         "pinhole-%hu", uid);
+#ifdef USE_LIBPFCTL
+	if(pfctl_get_rules_info(dev, &ri, PF_PASS, anchor_name) < 0)
+	{
+		syslog(LOG_ERR, "pfctl_get_rules_info: %m");
+		return -1;
+	}
+	n = ri.nr;
+#ifdef PF_RELEASETICKETS
+	tnum = ri.ticket;
+#endif /* PF_RELEASETICKETS */
+#else /* USE_LIBPFCTL */
 	memset(&pr, 0, sizeof(pr));
 	strlcpy(pr.anchor, anchor_name, MAXPATHLEN);
 #ifndef PF_NEWSTYLE
-	pr.rule.action = PF_PASS;
+	RULE.action = PF_PASS;
 #endif
 	if(ioctl(dev, DIOCGETRULES, &pr) < 0) {
 		syslog(LOG_ERR, "ioctl(dev, DIOCGETRULES, ...): %m");
 		return -1;
 	}
 	n = pr.nr;
+#ifdef PF_RELEASETICKETS
+	tnum = pr.ticket;
+#endif
+#endif /* USE_LIBPFCTL */
 	for(i=0; i<n; i++) {
+#ifdef USE_LIBPFCTL
+		if(pfctl_get_rule(dev, i, ri.ticket, anchor_name, PF_PASS, &rule, anchor_call) < 0)
+		{
+			syslog(LOG_ERR, "pfctl_get_rule: %m");
+			return -1;
+		}
+		strlcpy(tmp_label, RULE.label[0], sizeof(tmp_label));
+#else /* USE_LIBPFCTL */
 		pr.nr = i;
 		if(ioctl(dev, DIOCGETRULE, &pr) < 0) {
 			syslog(LOG_ERR, "ioctl(dev, DIOCGETRULE): %m");
 			return -1;
 		}
-		strlcpy(tmp_label, pr.rule.label, sizeof(tmp_label));
+		strlcpy(tmp_label, RULE.label, sizeof(tmp_label));
+#endif /* USE_LIBPFCTL */
 		strtok(tmp_label, " ");
 		if(0 == strcmp(tmp_label, label_start)) {
+#ifdef USE_LIBPFCTL
+			/* TODO: convert DIOCCHANGERULE to libpfctl */
+			struct pfioc_rule pr;
 			pr.action = PF_CHANGE_GET_TICKET;
+			memset(&pr, 0, sizeof(pr));
+			pr.ticket = ri.ticket;
+			pr.nr = i;
+			strlcpy(pr.anchor, anchor_name, MAXPATHLEN);
+#endif /* USE_LIBPFCTL */
 			if(ioctl(dev, DIOCCHANGERULE, &pr) < 0) {
 				syslog(LOG_ERR, "ioctl(dev, DIOCCHANGERULE, ...) PF_CHANGE_GET_TICKET: %m");
+				release_ticket(dev, tnum);
 				return -1;
 			}
 			pr.action = PF_CHANGE_REMOVE;
 			pr.nr = i;
 			if(ioctl(dev, DIOCCHANGERULE, &pr) < 0) {
 				syslog(LOG_ERR, "ioctl(dev, DIOCCHANGERULE, ...) PF_CHANGE_REMOVE: %m");
+				release_ticket(dev, tnum);
 				return -1;
 			}
+			release_ticket(dev, tnum);
 			return 0;
 		}
 	}
+	release_ticket(dev, tnum);
 	/* not found */
 	return -2;
 }
@@ -281,7 +380,17 @@ get_pinhole_info(unsigned short uid,
                  u_int64_t * packets, u_int64_t * bytes)
 {
 	int i, n;
+	unsigned int tnum;
+#undef RULE
+#ifdef USE_LIBPFCTL
+	struct pfctl_rules_info ri;
+	struct pfctl_rule rule;
+#define RULE (rule)
+	char anchor_call[MAXPATHLEN] = "";
+#else /* USE_LIBPFCTL */
 	struct pfioc_rule pr;
+#define RULE (pr.rule)
+#endif /* USE_LIBPFCTL */
 	char label_start[PF_RULE_LABEL_SIZE];
 	char tmp_label[PF_RULE_LABEL_SIZE];
 	char * p;
@@ -292,38 +401,66 @@ get_pinhole_info(unsigned short uid,
 	}
 	snprintf(label_start, sizeof(label_start),
 	         "pinhole-%hu", uid);
+#ifdef USE_LIBPFCTL
+	if(pfctl_get_rules_info(dev, &ri, PF_PASS, anchor_name) < 0)
+	{
+		syslog(LOG_ERR, "pfctl_get_rules_info: %m");
+		return -1;
+	}
+	n = ri.nr;
+#ifdef PF_RELEASETICKETS
+	tnum = ri.ticket;
+#endif /* PF_RELEASETICKETS */
+#else /* USE_LIBPFCTL */
 	memset(&pr, 0, sizeof(pr));
 	strlcpy(pr.anchor, anchor_name, MAXPATHLEN);
 #ifndef PF_NEWSTYLE
-	pr.rule.action = PF_PASS;
+	RULE.action = PF_PASS;
 #endif
 	if(ioctl(dev, DIOCGETRULES, &pr) < 0) {
 		syslog(LOG_ERR, "ioctl(dev, DIOCGETRULES, ...): %m");
 		return -1;
 	}
 	n = pr.nr;
+#ifdef PF_RELEASETICKETS
+	tnum = pr.ticket;
+#endif
+#endif /* USE_LIBPFCTL */
 	for(i=0; i<n; i++) {
+#ifdef USE_LIBPFCTL
+		if(pfctl_get_rule(dev, i, ri.ticket, anchor_name, PF_PASS, &rule, anchor_call) < 0)
+		{
+			syslog(LOG_ERR, "pfctl_get_rule: %m");
+			release_ticket(dev, tnum);
+			return -1;
+		}
+		strlcpy(tmp_label, RULE.label[0], sizeof(tmp_label));
+#else /* USE_LIBPFCTL */
 		pr.nr = i;
 		if(ioctl(dev, DIOCGETRULE, &pr) < 0) {
 			syslog(LOG_ERR, "ioctl(dev, DIOCGETRULE): %m");
+			release_ticket(dev, tnum);
 			return -1;
 		}
-		strlcpy(tmp_label, pr.rule.label, sizeof(tmp_label));
+		strlcpy(tmp_label, RULE.label, sizeof(tmp_label));
+#endif /* USE_LIBPFCTL */
 		p = tmp_label;
 		strsep(&p, " ");
 		if(0 == strcmp(tmp_label, label_start)) {
-			if(rem_host && (inet_ntop(AF_INET6, &pr.rule.src.addr.v.a.addr.v6, rem_host, rem_hostlen) == NULL)) {
+			if(rem_host && (inet_ntop(AF_INET6, &RULE.src.addr.v.a.addr.v6, rem_host, rem_hostlen) == NULL)) {
+				release_ticket(dev, tnum);
 				return -1;
 			}
 			if(rem_port)
-				*rem_port = ntohs(pr.rule.src.port[0]);
-			if(int_client && (inet_ntop(AF_INET6, &pr.rule.dst.addr.v.a.addr.v6, int_client, int_clientlen) == NULL)) {
+				*rem_port = ntohs(RULE.src.port[0]);
+			if(int_client && (inet_ntop(AF_INET6, &RULE.dst.addr.v.a.addr.v6, int_client, int_clientlen) == NULL)) {
+				release_ticket(dev, tnum);
 				return -1;
 			}
 			if(int_port)
-				*int_port = ntohs(pr.rule.dst.port[0]);
+				*int_port = ntohs(RULE.dst.port[0]);
 			if(proto)
-				*proto = pr.rule.proto;
+				*proto = RULE.proto;
 			if(timestamp)
 				sscanf(p, "ts-%u", timestamp);
 			if(desc) {
@@ -336,18 +473,20 @@ get_pinhole_info(unsigned short uid,
 			}
 #ifdef PFRULE_INOUT_COUNTS
 			if(packets)
-				*packets = pr.rule.packets[0] + pr.rule.packets[1];
+				*packets = RULE.packets[0] + RULE.packets[1];
 			if(bytes)
-				*bytes = pr.rule.bytes[0] + pr.rule.bytes[1];
+				*bytes = RULE.bytes[0] + RULE.bytes[1];
 #else
 			if(packets)
-				*packets = pr.rule.packets;
+				*packets = RULE.packets;
 			if(bytes)
-				*bytes = pr.rule.bytes;
+				*bytes = RULE.bytes;
 #endif
+			release_ticket(dev, tnum);
 			return 0;
 		}
 	}
+	release_ticket(dev, tnum);
 	/* not found */
 	return -2;
 }
@@ -368,9 +507,18 @@ int update_pinhole(unsigned short uid, unsigned int timestamp)
 int clean_pinhole_list(unsigned int * next_timestamp)
 {
 	int i;
+#undef RULE
+#ifdef USE_LIBPFCTL
+	struct pfctl_rules_info ri;
+	struct pfctl_rule rule;
+#define RULE (rule)
+	char anchor_call[MAXPATHLEN] = "";
+#else /* USE_LIBPFCTL */
 	struct pfioc_rule pr;
+#define RULE (pr.rule)
+#endif /* USE_LIBPFCTL */
 	time_t current_time;
-	unsigned int ts;
+	unsigned int ts, tnum;
 	int uid;
 	unsigned int min_ts = UINT_MAX;
 	int min_uid = INT_MAX, max_uid = -1;
@@ -381,46 +529,104 @@ int clean_pinhole_list(unsigned int * next_timestamp)
 		return -1;
 	}
 	current_time = upnp_time();
+#ifdef USE_LIBPFCTL
+	if(pfctl_get_rules_info(dev, &ri, PF_PASS, anchor_name) < 0)
+	{
+		syslog(LOG_ERR, "pfctl_get_rules_info: %m");
+		return -1;
+	}
+	i = ri.nr - 1;
+#ifdef PF_RELEASETICKETS
+	tnum = ri.ticket;
+#endif /* PF_RELEASETICKETS */
+#else /* USE_LIBPFCTL */
 	memset(&pr, 0, sizeof(pr));
 	strlcpy(pr.anchor, anchor_name, MAXPATHLEN);
 #ifndef PF_NEWSTYLE
-	pr.rule.action = PF_PASS;
+	RULE.action = PF_PASS;
 #endif
 	if(ioctl(dev, DIOCGETRULES, &pr) < 0) {
 		syslog(LOG_ERR, "ioctl(dev, DIOCGETRULES, ...): %m");
 		return -1;
 	}
-	for(i = pr.nr - 1; i >= 0; i--) {
+	i = pr.nr - 1;
+#ifdef PF_RELEASETICKETS
+	tnum = pr.ticket;
+#endif
+#endif /* USE_LIBPFCTL */
+	for(; i >= 0; i--) {
+#ifdef USE_LIBPFCTL
+		if(pfctl_get_rule(dev, i, ri.ticket, anchor_name, PF_PASS, &rule, anchor_call) < 0)
+		{
+			syslog(LOG_ERR, "pfctl_get_rule: %m");
+			release_ticket(dev, tnum);
+			return -1;
+		}
+		if(sscanf(RULE.label[0], PINEHOLE_LABEL_FORMAT_SKIPDESC, &uid, &ts) != 2) {
+			syslog(LOG_DEBUG, "rule with label '%s' is not a IGD pinhole", RULE.label[0]);
+			continue;
+		}
+#else /* USE_LIBPFCTL */
 		pr.nr = i;
 		if(ioctl(dev, DIOCGETRULE, &pr) < 0) {
 			syslog(LOG_ERR, "ioctl(dev, DIOCGETRULE): %m");
+			release_ticket(dev, tnum);
 			return -1;
 		}
-		if(sscanf(pr.rule.label, PINEHOLE_LABEL_FORMAT_SKIPDESC, &uid, &ts) != 2) {
-			syslog(LOG_DEBUG, "rule with label '%s' is not a IGD pinhole", pr.rule.label);
+		if(sscanf(RULE.label, PINEHOLE_LABEL_FORMAT_SKIPDESC, &uid, &ts) != 2) {
+			syslog(LOG_DEBUG, "rule with label '%s' is not a IGD pinhole", RULE.label);
 			continue;
 		}
+#endif /* USE_LIBPFCTL */
 		if(ts <= (unsigned int)current_time) {
-			syslog(LOG_INFO, "removing expired pinhole '%s'", pr.rule.label);
+#ifdef USE_LIBPFCTL
+			/* TODO: convert DIOCCHANGERULE to libpfctl */
+			struct pfioc_rule pr;
+			pr.action = PF_CHANGE_GET_TICKET;
+			memset(&pr, 0, sizeof(pr));
+			pr.ticket = ri.ticket;
+			pr.nr = i;
+			strlcpy(pr.anchor, anchor_name, MAXPATHLEN);
+			syslog(LOG_INFO, "removing expired pinhole '%s'", RULE.label[0]);
+#else /* USE_LIBPFCTL */
+			syslog(LOG_INFO, "removing expired pinhole '%s'", RULE.label);
+#endif /* USE_LIBPFCTL */
 			pr.action = PF_CHANGE_GET_TICKET;
 			if(ioctl(dev, DIOCCHANGERULE, &pr) < 0) {
 				syslog(LOG_ERR, "ioctl(dev, DIOCCHANGERULE, ...) PF_CHANGE_GET_TICKET: %m");
+				release_ticket(dev, tnum);
 				return -1;
 			}
 			pr.action = PF_CHANGE_REMOVE;
 			pr.nr = i;
 			if(ioctl(dev, DIOCCHANGERULE, &pr) < 0) {
 				syslog(LOG_ERR, "ioctl(dev, DIOCCHANGERULE, ...) PF_CHANGE_REMOVE: %m");
+				release_ticket(dev, tnum);
 				return -1;
 			}
 			n++;
+#ifdef USE_LIBPFCTL
+			if(pfctl_get_rules_info(dev, &ri, PF_PASS, anchor_name) < 0)
+			{
+				syslog(LOG_ERR, "pfctl_get_rules_info: %m");
+				return -1;
+			}
+#ifdef PF_RELEASETICKETS
+			tnum = ri.ticket;
+#endif /* PF_RELEASETICKETS */
+#else /* USE_LIBPFCTL */
 #ifndef PF_NEWSTYLE
-			pr.rule.action = PF_PASS;
+			RULE.action = PF_PASS;
 #endif
+			release_ticket(dev, tnum);
 			if(ioctl(dev, DIOCGETRULES, &pr) < 0) {
 				syslog(LOG_ERR, "ioctl(dev, DIOCGETRULES, ...): %m");
 				return -1;
 			}
+#ifdef PF_RELEASETICKETS
+			tnum = pr.ticket;
+#endif
+#endif /* USE_LIBPFCTL */
 		} else {
 			if(uid > max_uid)
 				max_uid = uid;
@@ -440,6 +646,7 @@ int clean_pinhole_list(unsigned int * next_timestamp)
 			next_uid = 1;
 		}
 	}
+	release_ticket(dev, tnum);
 	return n;	/* number of rules removed */
 }
 
