@@ -1,5 +1,5 @@
 #!/bin/sh
-# $Id: testgetifaddr.sh,v 1.7 2021/11/09 17:50:43 nanard Exp $
+# $Id: testgetifaddr.sh,v 1.8 2025/04/09 22:49:37 nanard Exp $
 
 OS=`uname -s`
 case $OS in
@@ -10,6 +10,7 @@ case $OS in
 	EXTIP="`$IFCONFIG $EXTIF | awk '/inet / { print $2 }' `"
 	;;
 	*)
+	# linux
 	IP="`command -v ip`" || exit 1
 	EXTIF="`LC_ALL=C $IP -4 route | grep 'default' | sed -e 's/.*dev[[:space:]]*//' -e 's/[[:space:]].*//'`" || exit 1
 	EXTIF="`LC_ALL=C $IP -4 addr show $EXTIF | awk '/[0-9]+:/ { print $2; exit 0 }' | cut -d ":" -f 1`"
@@ -18,13 +19,16 @@ case $OS in
 esac
 
 #echo "Interface : $EXTIF   IP address : $EXTIP"
-RES=`./testgetifaddr $EXTIF | head -n1 | sed 's/Interface .* has IP address \(.*\)\./\1/'` || exit 1
+STATUS=`./testgetifaddr $EXTIF | grep status | sed 's/^.*status : \(.*\)/\1/'` || exit 1
+RES=`./testgetifaddr $EXTIF | grep 'has IP address' | sed 's/Interface .* has IP address \(.*\)\./\1/'` || exit 1
 
-
-if [ "$RES" = "$EXTIP" ] ; then
-	echo "testgetifaddr test OK"
-	exit 0
-else
+if [ "$RES" != "$EXTIP" ] ; then
 	echo "testgetifaddr test FAILED : $EXTIP != $RES"
 	exit 1
+elif [ "$STATUS" != "Connected" ] ; then
+	echo "testgetifaddr test FAILED : status is $STATUS"
+	exit 1
+else
+	echo "testgetifaddr test OK"
+	exit 0
 fi
