@@ -1,4 +1,4 @@
-/* $Id: testgetifstats.c,v 1.5 2012/03/05 20:36:17 nanard Exp $ */
+/* $Id: testgetifstats.c,v 1.6 2012/06/22 16:11:58 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
  * (c) 2006 Thomas Bernard
@@ -11,6 +11,11 @@
 
 #include "getifstats.h"
 
+#if defined(__sun)
+/* solaris 10 does not define LOG_PERROR */
+#define LOG_PERROR 0
+#endif
+
 int
 main(int argc, char **argv)
 {
