@@ -1,7 +1,7 @@
-/* $Id: testportinuse.c,v 1.5 2020/09/28 21:49:18 nanard Exp $ */
+/* $Id: testportinuse.c,v 1.6 2025/04/21 21:48:56 nanard Exp $ */
 /* MiniUPnP project
- * (c) 2014 Thomas Bernard
- * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
+ * (c) 2014-2025 Thomas Bernard
+ * http://miniupnp.free.fr/ or https://miniupnp.tuxfamily.org/
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 #include <stdio.h>
@@ -13,6 +13,10 @@
 #include "macros.h"
 #include "config.h"
 #include "portinuse.h"
+#include "upnpglobalvars.h"
+
+struct lan_addr_list lan_addrs;
+time_t startup_time = 0;
 
 int main(int argc, char * * argv)
 {
