@@ -1,8 +1,8 @@
-/* $Id: testupnpdescgen.c,v 1.37 2020/11/04 21:02:29 nanard Exp $ */
+/* $Id: testupnpdescgen.c,v 1.38 2021/08/21 08:16:16 nanard Exp $ */
 /* vim: tabstop=4 shiftwidth=4 noexpandtab
  * MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
- * (c) 2006-2020 Thomas Bernard
+ * (c) 2006-2021 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -142,6 +142,7 @@ void stupid_test(void)
 int
 main(int argc, char * * argv)
 {
+	int force_igd1 = 0;
 	char * rootDesc;
 	int rootDescLen;
 	char * s;
@@ -160,7 +161,7 @@ main(int argc, char * * argv)
 			return 0;
 #ifdef IGD_V2
 		} else if(0 == strcmp(argv[l], "--forceigdv1")) {
-			SETFLAG(FORCEIGDDESCV1MASK);
+			force_igd1 = 1;
 #endif
 		} else {
 			fprintf(stderr, "unknown option %s\n", argv[l]);
@@ -173,7 +174,7 @@ main(int argc, char * * argv)
 		}
 	}
 	printf("Root Description :\n");
-	rootDesc = genRootDesc(&rootDescLen);
+	rootDesc = genRootDesc(&rootDescLen, force_igd1);
 	xml_pretty_print(rootDesc, rootDescLen, stdout);
 	f = fopen("testdescs/rootdesc.xml", "w");
 	if(f) {
@@ -183,7 +184,7 @@ main(int argc, char * * argv)
 	free(rootDesc);
 	printf("\n-------------\n");
 	printf("WANIPConnection Description :\n");
-	s = genWANIPCn(&l);
+	s = genWANIPCn(&l, force_igd1);
 	xml_pretty_print(s, l, stdout);
 	f = fopen("testdescs/wanipc_scpd.xml", "w");
 	if(f) {
@@ -193,7 +194,7 @@ main(int argc, char * * argv)
 	free(s);
 	printf("\n-------------\n");
 	printf("WANConfig Description :\n");
-	s = genWANCfg(&l);
+	s = genWANCfg(&l, force_igd1);
 	xml_pretty_print(s, l, stdout);
 	f = fopen("testdescs/wanconfig_scpd.xml", "w");
 	if(f) {
@@ -204,7 +205,7 @@ main(int argc, char * * argv)
 	printf("\n-------------\n");
 #ifdef ENABLE_L3F_SERVICE
 	printf("Layer3Forwarding service :\n");
-	s = genL3F(&l);
+	s = genL3F(&l, force_igd1);
 	xml_pretty_print(s, l, stdout);
 	f = fopen("testdescs/l3f_scpd.xml", "w");
 	if(f) {
@@ -216,7 +217,7 @@ main(int argc, char * * argv)
 #endif
 #ifdef ENABLE_6FC_SERVICE
 	printf("WANIPv6FirewallControl service :\n");
-	s = gen6FC(&l);
+	s = gen6FC(&l, force_igd1);
 	xml_pretty_print(s, l, stdout);
 	f = fopen("testdescs/wanipv6fc_scpd.xml", "w");
 	if(f) {
@@ -228,7 +229,7 @@ main(int argc, char * * argv)
 #endif
 #ifdef ENABLE_DP_SERVICE
 	printf("DeviceProtection service :\n");
-	s = genDP(&l);
+	s = genDP(&l, force_igd1);
 	xml_pretty_print(s, l, stdout);
 	f = fopen("testdescs/dp_scpd.xml", "w");
 	if(f) {
