@@ -1,8 +1,8 @@
-/* $Id: upnpglobalvars.h,v 1.52 2021/08/21 08:15:00 nanard Exp $ */
+/* $Id: upnpglobalvars.h,v 1.55 2025/04/21 21:51:05 nanard Exp $ */
 /* vim: tabstop=4 shiftwidth=4 noexpandtab
  * MiniUPnP project
  * http://miniupnp.free.fr/ or https://miniupnp.tuxfamily.org/
- * (c) 2006-2021 Thomas Bernard
+ * (c) 2006-2025 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
@@ -87,6 +87,9 @@ extern int runtime_flags;
 
 #define PERFORMSTUNMASK    0x1000
 
+#define ALLOWPRIVATEIPV4MASK 0x2000
+#define ALLOWFILTEREDSTUNMASK 0x4000
+
 #define SETFLAG(mask)	runtime_flags |= mask
 #define GETFLAG(mask)	(runtime_flags & mask)
 #define CLEARFLAG(mask)	runtime_flags &= ~mask
