@@ -1,4 +1,4 @@
-/* $Id: upnpredirect.c,v 1.101 2025/03/22 22:19:33 nanard Exp $ */
+/* $Id: upnpredirect.c,v 1.102 2025/04/12 23:14:31 nanard Exp $ */
 /* vim: tabstop=4 shiftwidth=4 noexpandtab
  * MiniUPnP project
  * http://miniupnp.free.fr/ or https://miniupnp.tuxfamily.org/
@@ -51,45 +51,6 @@
 #define PRIu64 "llu"
 #endif
 
-/* proto_atoi()
- * convert the string "UDP" or "TCP" to IPPROTO_UDP and IPPROTO_UDP */
-static int
-proto_atoi(const char * protocol)
-{
-	int proto = IPPROTO_TCP;
-	if(strcasecmp(protocol, "UDP") == 0)
-		proto = IPPROTO_UDP;
-#ifdef IPPROTO_UDPLITE
-	else if(strcasecmp(protocol, "UDPLITE") == 0)
-		proto = IPPROTO_UDPLITE;
-#endif /* IPPROTO_UDPLITE */
-	return proto;
-}
-
-/* proto_itoa()
- * convert IPPROTO_UDP, IPPROTO_UDP, etc. to "UDP", "TCP" */
-static const char *
-proto_itoa(int proto)
-{
-	const char * protocol;
-	switch(proto) {
-	case IPPROTO_UDP:
-		protocol = "UDP";
-		break;
-	case IPPROTO_TCP:
-		protocol = "TCP";
-		break;
-#ifdef IPPROTO_UDPLITE
-	case IPPROTO_UDPLITE:
-		protocol = "UDPLITE";
-		break;
-#endif /* IPPROTO_UDPLITE */
-	default:
-		protocol = "*UNKNOWN*";
-	}
-	return protocol;
-}
-
 #ifdef ENABLE_LEASEFILE
 static int
 lease_file_add(unsigned short eport,
