@@ -1,12 +1,12 @@
-/* $Id: upnpsoap.h,v 1.9 2012/03/05 20:36:19 nanard Exp $ */
+/* $Id: upnpsoap.h,v 1.10 2012/09/27 15:46:18 nanard Exp $ */
 /* MiniUPnP project
  * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
  * (c) 2006 Thomas Bernard
  * This software is subject to the conditions detailed
  * in the LICENCE file provided within the distribution */
 
-#ifndef __UPNPSOAP_H__
-#define __UPNPSOAP_H__
+#ifndef UPNPSOAP_H_INCLUDED
+#define UPNPSOAP_H_INCLUDED
 
 /* ExecuteSoapAction():
  * this method executes the requested Soap Action */
